import socket
import ssl

HOST, PORT = '0.0.0.0', 8443
ADDR = (HOST, PORT)


# make a simple SSL context with a self-signed localhost pair
def make_context():
    ctx = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH)
    ctx.load_cert_chain('localhost.cert', 'localhost.pkey')
    return ctx


# msg_callback that does nothing
def msg_callback(conn, direction, version, content_type, msg_type, data):
    print('msg_callback')


# sni_callback that attaches a new context to the socket
def sni_callback(sslsocket, sni_name, sslcontext):
    print('sni_callback')
    new_ctx = make_context()
    # uncomment this line to "fix" the deadlock
#    new_ctx._msg_callback = msg_callback
    sslsocket.context = new_ctx


# listen, accept, and close connections from client
#
# trigger the deadlock via:
# openssl s_client -connect localhost:8443
if __name__ == '__main__':
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(ADDR)
    s.listen()

    ctx = make_context()
    ctx._msg_callback = msg_callback
    ctx.sni_callback = sni_callback

    s = ctx.wrap_socket(s, server_side=True)

    print('listening on:', ADDR)
    while True:
        c, _ = s.accept()
        c.close()
